@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2007 Henrique Pinto <henrique.pinto@kdemail.net>
  * Copyright (C) 2008-2009 Harald Hvaal <haraldhv@stud.ntnu.no>
- * Copyright (C) 2009 Raphael Kubo da Costa <kubito@gmail.com>
+ * Copyright (C) 2009-2011 Raphael Kubo da Costa <rakuco@FreeBSD.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -60,6 +60,7 @@
 #include <QMenu>
 #include <QMimeData>
 #include <QMouseEvent>
+#include <QScopedPointer>
 #include <QSplitter>
 #include <QTimer>
 #include <QVBoxLayout>
@@ -78,7 +79,6 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
         : KParts::ReadWritePart(parent),
           m_model(new ArchiveModel(this)),
           m_splitter(0),
-          m_previewDir(0),
           m_busy(false),
           m_jobTracker(0)
 {
@@ -130,12 +130,9 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
 
 Part::~Part()
 {
-    updateSplitterSizes();
+    saveSplitterSizes();
 
     m_extractFilesAction->menu()->deleteLater();
-
-    delete m_previewDir;
-    m_previewDir = 0;
 }
 
 void Part::registerJob(KJob* job)
@@ -397,7 +394,7 @@ bool Part::openFile()
         }
     }
 
-    Kerfuffle::Archive *archive = Kerfuffle::factory(localFile);
+    QScopedPointer<Kerfuffle::Archive> archive(Kerfuffle::Archive::create(localFile, m_model));
 
     if ((!archive) || ((creatingNewArchive) && (archive->isReadOnly()))) {
         QStringList mimeTypeList;
@@ -440,7 +437,7 @@ bool Part::openFile()
             return false;
         }
 
-        archive = Kerfuffle::factory(localFile, mimeTypes.key(item));
+        archive.reset(Kerfuffle::Archive::create(localFile, mimeTypes.key(item), m_model));
     }
 
     if (!archive) {
@@ -448,7 +445,7 @@ bool Part::openFile()
         return false;
     }
 
-    KJob *job = m_model->setArchive(archive);
+    KJob *job = m_model->setArchive(archive.take());
     registerJob(job);
     job->start();
     m_infoPanel->setIndex(QModelIndex());
@@ -528,10 +525,6 @@ void Part::slotPreview()
 
 void Part::slotPreview(const QModelIndex & index)
 {
-    if (!m_previewDir) {
-        m_previewDir = new KTempDir();
-    }
-
     if (!isPreviewable(index)) {
         return;
     }
@@ -542,7 +535,7 @@ void Part::slotPreview(const QModelIndex & index)
         Kerfuffle::ExtractionOptions options;
         options[QLatin1String( "PreservePaths" )] = true;
 
-        ExtractJob *job = m_model->extractFile(entry[ InternalID ], m_previewDir->name(), options);
+        ExtractJob *job = m_model->extractFile(entry[ InternalID ], m_previewDir.name(), options);
         registerJob(job);
         connect(job, SIGNAL(result(KJob*)),
                 this, SLOT(slotPreviewExtracted(KJob*)));
@@ -560,7 +553,7 @@ void Part::slotPreviewExtracted(KJob *job)
             m_model->entryForIndex(m_view->selectionModel()->currentIndex());
 
         QString fullName =
-            m_previewDir->name() + QLatin1Char('/') + entry[FileName].toString();
+            m_previewDir.name() + QLatin1Char('/') + entry[FileName].toString();
 
         // Make sure a maliciously crafted archive with parent folders named ".." do
         // not cause the previewed file path to be located outside the temporary
@@ -854,10 +847,10 @@ void Part::slotToggleInfoPanel(bool visible)
     }
 
     m_splitter->setSizes(splitterSizes);
-    updateSplitterSizes();
+    saveSplitterSizes();
 }
 
-void Part::updateSplitterSizes()
+void Part::saveSplitterSizes()
 {
     ArkSettings::setSplitterSizes(m_splitter->sizes());
     ArkSettings::self()->writeConfig();
